/*   Allows users to find out various info about their ACUA accounts.
 *   Telnet Server Edition.
 *   Copyright (C) 1999 Robert Davidson.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   Written by Robert Davidson, E-Mail: security@mail.ebbs.com.au
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <syslog.h>
#include <netinet/in.h>
#include <time.h>
#include "radutmp.h"

#define Version 	"1.02-13-02-2001"
#define ACUA_USERS 	"/usr/lib/acua/acua_users"
#define CFG_FILE 	"/usr/local/etc/acua_ui.conf"
#define ACUA     	"/usr/bin/acua"

// How long to wait (in seconds) between updates.
#define WAIT_TIME 55

#ifndef RADUTMP
#define RADUTMP 	"/var/log/radutmp"
#endif

/* Define this if you want logging enabled. Logging is done via syslog() */
/* Set the below to be "1", otherwise "0" to not log anything. */
#define LOGGING                  1

/* Logging. 0=On, 1=Off. */
#ifndef LOGGING
#define LOGGING 0
#endif

// end of configurable stuff.

/* Procedure: UpdateStats()
 * Returns: 0=Ok, 1=Error
 * Description: This procedure will read the ACUA users file and determine
 *              certain statistics for the users, then print them to stdout.
 */
int UpdateStats(char *un, char *tf)
{
char junk[256];
FILE *fp;

sprintf(junk, "%s pp %s %s %s", ACUA, un, CFG_FILE, tf);
if (system(junk)) { if (LOGGING) syslog(LOG_ERR, "Unable to execute: %s", junk);
                    exit(1);
}

// print the contents of the output file to stdout.

fp=fopen(tf, "r");

if (!fp) { /* File open failed!! */
           printf("ERROR: Unable to open %s for reading.", tf);
           fflush(stdout);
           if (LOGGING) syslog(LOG_ERR, "Unable to open %s for reading.", tf);
           return(1);
         }

while (!feof(fp)) { /* Read line by line of the file and dump it to stdout */
                    fgets(junk, sizeof(junk), fp);
                    if (!feof(fp)) {
                     if (strlen(junk)>0) junk[strlen(junk)-1]='\0';  /* Strip the CR/LF */
                     printf("%s\r\n", junk);
                     fflush(stdout);
                    }
                  }
fclose(fp);
// Delete the tempory file we made.
if (unlink(tf)!=0)
  if (LOGGING) syslog(LOG_ERR, "Could not delete tempory file: %s", tf);
return(0);
}

/* ----------------------------------------------------------------------- */
/* - Start Of Program. --------------------------------------------------- */
/* ----------------------------------------------------------------------- */

int main()
{

char   UserName[35]="", fa[25], ia[25], tf[21];
struct sockaddr_in sai;
struct radutmp     rt;
FILE   *fp;
size_t sv = sizeof(sai);
int a=0;

strcpy(tf, "/tmp/acua_ui.XXXXXX");
mktemp(tf);

/* Open the syslog */
if (LOGGING) openlog("acua_ui_ts", LOG_PID, LOG_USER);

/* Change directory to our tempory directory */
if (chdir("/tmp")!=0) { /* Error doing chdir */
                        fflush(stdout);
                        if (LOGGING) syslog(LOG_ERR, "Can not change to directory: /tmp");
                          exit(1);
                        }

/* Figure out the remote IP Address */
if (getpeername(0, (struct sockaddr *) &sai, &sv) < 0)
     { printf("\ngetpeername() Failed!\n");
       fflush(stdout);
       exit(1);
     }

// From their IP Address, figure out who the user is..
if ((fp = fopen(RADUTMP, "r")) == NULL) {
  printf("Error opening file.\n");
  fflush(stdout);
  if (LOGGING) syslog(LOG_ERR, "Can't open file for reading: %s", RADUTMP);
  exit(1);
}

// read and compare the logins.
UserName[0] = '\0';
while(fread(&rt, sizeof(rt), 1, fp) == 1) {
  // if they match and are logged on...
  if (rt.type == 1) {
    sprintf(fa, "%s", inet_ntoa(rt.framed_address));
    sprintf(ia, "%s", inet_ntoa(sai.sin_addr));
    if (strcmp(fa, ia) == 0) sprintf(UserName, "%s", rt.login);
  }
}

// if we didn't find a match..
if (UserName[0] == '\0') { printf("Sorry, the information you have requested is not currently available.\n");
                           fflush(stdout);
                           sleep(8);
                           exit(0);
}

if (LOGGING) syslog(LOG_INFO, "Connection From: [%s] User: [%s]", inet_ntoa(sai.sin_addr), UserName);

// Time to go into a little loop now..
for (;;) { if (UpdateStats(UserName, tf)==1) exit(1); // error.
           sleep(WAIT_TIME);
}

return(0);
}

